
/* 	Copyright John D. "smeezekitty" 2017
	
	FreeNBTV is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    FreeNBTV is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with FreeNBTV.  If not, see <http://www.gnu.org/licenses/>.
*/

/* This file handles the writing of MS Wave files*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <gtk/gtk.h>
#include <SDL/SDL.h>
#include <SDL/SDL_image.h>
#include "nbtv_format.h"
#include "output.h"
#include "input.h"
#include "nbtv_gen.h"
#include "gui.h"

/*Header setup for PCM*/
char header_preload[] = {'R', 'I', 'F', 'F', 0, 0, 0, 0, 'W', 'A', 'V', 'E', 'f', 'm', 't', ' ', 16, 0, 0, 0, 1, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 'd', 'a', 't', 'a'};

void ms_wave_write_start(struct _output *op){
	if(!op->file){
		return;
	}
	if(!op->header){
		op->header = malloc(sizeof(struct WAVE_header));
	}
	memcpy(op->header, header_preload, sizeof(header_preload));
	op->header->SampleRate = op->sample_rate;
	op->header->channels = op->num_channels;
	op->header->ByteRate = op->sample_rate * op->num_channels * (op->bps>>3);
	op->header->BlockAlign = op->header->ByteRate/op->sample_rate;
	op->header->bps = op->bps;
	op->header->ChunkSize = 36;
	op->header->alength = 0;
	if(!op->header->ByteRate){
		op->header->ByteRate++;
	}
	/*Write the header now even though not all fields are set up to reserve space at the beginning of the file*/
	fwrite(op->header, sizeof(struct WAVE_header), 1, op->file);
}
void audio_file_write_block(struct _output *op, uint8_t *samples, int count){
	int data_len = count * op->num_channels * op->bps / 8;
	if(!op->file){
		return;
	}
	if(op->header){
		op->header->alength += data_len;
		op->header->ChunkSize += data_len;
	}
	fwrite(samples, 1, data_len, op->file);
}
void audio_file_end(struct _output *op){
	if(!op->file){
		return;
	}
	if(op->header){
		fseek(op->file, 0, 0);
		fwrite(op->header, sizeof(struct WAVE_header), 1, op->file);
		free(op->header);
		op->header = NULL;
	}
	fclose(op->file);
	op->file = NULL;
}
	